@@ -33,10 +33,11 @@ from di._utils.topsort import topsort
 from di._utils.types import FusedContextManager
 from di.api.dependencies import DependantBase, DependencyParameter
 from di.api.executor import AsyncExecutor, SyncExecutor
-from di.api.providers import DependencyProvider, DependencyProviderType, DependencyType
+from di.api.providers import DependencyProvider, DependencyProviderType
 from di.api.scopes import Scope
 from di.api.solved import SolvedDependant
-from di.exceptions import SolvingError
+from di.dependant import Dependant
+from di.exceptions import SolvingError, WiringError
 from di.executors import DefaultExecutor
 
 _Task = Union[AsyncTask, SyncTask]
@@ -47,6 +48,9 @@ _ExecutionCM = FusedContextManager[None]
 _nullcontext = nullcontext(None)
 
 
+DependencyType = TypeVar("DependencyType")
+
+
 __all__ = ("BaseContainer", "Container")
 
 
@@ -81,10 +85,12 @@ class _ContainerCommon:
 
     def bind(
         self,
-        provider: DependantBase[DependencyType],
+        provider: Union[
+            DependantBase[DependencyType], DependencyProviderType[DependencyType]
+        ],
         dependency: DependencyProviderType[DependencyType],
     ) -> ContextManager[None]:
-        """Bind a new dependency provider for a given dependency.
+        """Replace a dependency provider with a new one.
 
         This can be used as a function (for a permanent bind, cleared when `scope` is exited)
         or as a context manager (the bind will be cleared when the context manager exits).
@@ -94,6 +100,9 @@ class _ContainerCommon:
         """
         previous_provider = self.binds.get(dependency, None)
 
+        if not isinstance(provider, DependantBase):
+            provider = Dependant(provider)
+
         self._binds[dependency] = provider
 
         @contextmanager
@@ -137,12 +146,29 @@ class _ContainerCommon:
             # exist as a bound value
             params = dep.get_dependencies().copy()
             for idx, param in enumerate(params):
-                assert param.dependency.call is not None
-                if param.dependency.call in self._binds:
-                    params[idx] = DependencyParameter(
-                        dependency=self._binds[param.dependency.call],
-                        parameter=param.parameter,
+                if param.parameter is not None:
+                    param = param._replace(
+                        dependency=param.dependency.register_parameter(param.parameter)
                     )
+                if (
+                    param.dependency.call is not None
+                    and param.dependency.call in self._binds
+                ):
+                    param = param._replace(
+                        dependency=self._binds[param.dependency.call]
+                    )
+                params[idx] = param
+                if param.parameter is not None:
+                    if (
+                        param.dependency.call is None
+                        and param.parameter.default is param.parameter.empty
+                    ):
+                        raise WiringError(
+                            f"The parameter {param.parameter.name} to {dep.call} has no dependency marker,"
+                            " no type annotation and no default value."
+                            " This will produce a TypeError when this function is called."
+                            " You must either provide a dependency marker, a type annotation or a default value."
+                        )
             return params
 
         def check_equivalent(dep: DependantBase[Any]) -> None:
@@ -174,21 +200,37 @@ class _ContainerCommon:
                     dep_dag[dep].append(predecessor_dep)
                     if predecessor_dep not in dep_registry:
                         q.append(predecessor_dep)
-
+        # Filter out any dependencies that do not have a call
+        # These do not become tasks since they don't need to be computed
+        computable_param_graph = {
+            dep: [
+                param for param in param_graph[dep] if param.dependency.call is not None
+            ]
+            for dep in param_graph
+            if dep.call is not None
+        }
         # Order the Dependant's topologically so that we can create Tasks
         # with references to all of their children
-        dep_topsort = topsort(dep_dag)
+        dep_topsort = topsort(
+            {
+                dep: [p.dependency for p in computable_param_graph[dep]]
+                for dep in computable_param_graph
+            }
+        )
         # Create a seperate TopologicalSorter to hold the Tasks
         ts: TopologicalSorter[_Task] = TopologicalSorter()
         tasks = self._build_tasks(
-            param_graph, (node for nodes in dep_topsort for node in nodes), ts
+            computable_param_graph,
+            (node for nodes in dep_topsort for node in nodes),
+            ts,
         )
-        for dep, predecessors in dep_dag.items():
-            ts.add(tasks[dep], *(tasks[p] for p in predecessors))
         ts.prepare()
         task_dependency_dag: _DependantTaskDag = {
-            tasks[dep]: {tasks[predecessor_dep] for predecessor_dep in predecessor_deps}
-            for dep, predecessor_deps in dep_dag.items()
+            tasks[dep]: {
+                tasks[predecessor_dep.dependency]
+                for predecessor_dep in predecessor_deps
+            }
+            for dep, predecessor_deps in computable_param_graph.items()
         }
         call_map: Dict[DependencyProvider, Set[_Task]] = {}
         for t in task_dependency_dag:
@@ -200,10 +242,12 @@ class _ContainerCommon:
             topological_sorter=ts,
             cacheable_tasks={
                 tasks[d]
-                for d in dep_dag
+                for d in computable_param_graph
                 if d.scope != self._execution_scope and d.share
             },
-            cached_tasks=tuple({(tasks[d], d.scope) for d in dep_dag if d.share}),
+            cached_tasks=tuple(
+                {(tasks[d], d.scope) for d in computable_param_graph if d.share}
+            ),
             validated_scopes=set(),
             call_map={k: tuple(v) for k, v in call_map.items()},
         )
@@ -229,10 +273,8 @@ class _ContainerCommon:
             positional: List[_Task] = []
             keyword: Dict[str, _Task] = {}
             for param in dag[dep]:
-                if param.parameter is not None and param.dependency.call is not None:
+                if param.parameter is not None:
                     task = tasks[param.dependency]
-                    # prefer positional arguments since those can be unpacked from a generator
-                    # saving generation of an intermediate dict
                     if param.parameter.kind is param.parameter.KEYWORD_ONLY:
                         keyword[param.parameter.name] = task
                     else:
@@ -340,15 +382,13 @@ class BaseContainer(_ContainerCommon):
         *,
         execution_scope: Scope = None,
         executor: Optional[Union[AsyncExecutor, SyncExecutor]] = None,
-        _state: Optional[ContainerState] = None,
-        _binds: Optional[Dict[DependencyProvider, DependantBase[Any]]] = None,
     ) -> None:
         super().__init__(
             executor=executor or DefaultExecutor(),
             execution_scope=execution_scope,
-            binds=_binds,
+            binds={},
         )
-        self.__state = _state or ContainerState.initialize()
+        self.__state = ContainerState.initialize()
 
     @property
     def binds(self) -> Mapping[DependencyProvider, DependantBase[Any]]:
@@ -363,13 +403,14 @@ class BaseContainer(_ContainerCommon):
         return self.__state
 
     def copy(self: _BaseContainerType) -> _BaseContainerType:
-        new = self.__class__(
-            execution_scope=self._execution_scope,
-            executor=self._executor,
-            _state=self.__state.copy(),  # cached values and scopes are not shared
-            _binds=self._binds,  # binds are shared
-        )
-        return new
+        new = object.__new__(self.__class__)
+        new._execution_scope = self._execution_scope
+        new._executor = self._executor
+        # binds are shared
+        new._binds = self._binds
+        # cached values and scopes are not shared
+        new.__state = self.__state.copy()
+        return new  # type: ignore[no-any-return]
 
     def enter_scope(
         self: _BaseContainerType, scope: Scope
@@ -435,19 +476,14 @@ class Container(_ContainerCommon):
         *,
         execution_scope: Scope = None,
         executor: Optional[Union[AsyncExecutor, SyncExecutor]] = None,
-        _context: Optional[contextvars.ContextVar[ContainerState]] = None,
-        _binds: Optional[Dict[DependencyProvider, DependantBase[Any]]] = None,
     ) -> None:
         super().__init__(
             executor=executor or DefaultExecutor(),
             execution_scope=execution_scope,
-            binds=_binds,
+            binds={},
         )
-        if _context is None:
-            self._context = contextvars.ContextVar(f"{self}._context")
-            self._context.set(ContainerState.initialize())
-        else:
-            self._context = _context
+        self._context = contextvars.ContextVar(f"{self}._context")
+        self._context.set(ContainerState.initialize())
 
     @property
     def _state(self) -> ContainerState:
@@ -458,13 +494,12 @@ class Container(_ContainerCommon):
         return self._state.stacks.keys()
 
     def copy(self: _ContainerType) -> _ContainerType:
-        new = type(self)(
-            execution_scope=self._execution_scope,
-            executor=self._executor,
-            _binds=self._binds,
-            _context=self._context,
-        )
-        return new
+        new = object.__new__(self.__class__)
+        new._execution_scope = self._execution_scope
+        new._executor = self._executor
+        new._binds = self._binds
+        new._context = self._context
+        return new  # type: ignore[no-any-return]
 
     def enter_scope(
         self: _ContainerType, scope: Scope
