@@ -1,19 +1,20 @@
 """Convenience functions, mainly for the purpose of providing proper type annotations for default arguments.
 """
 
-from typing import Optional, overload
+from typing import Optional, TypeVar, overload
 
 from di.api.providers import (
     AsyncGeneratorProvider,
     CallableProvider,
     CoroutineProvider,
     DependencyProviderType,
-    DependencyType,
     GeneratorProvider,
 )
 from di.api.scopes import Scope
 from di.dependant import Dependant
 
+DependencyType = TypeVar("DependencyType")
+
 
 @overload
 def Depends(
@@ -22,7 +23,6 @@ def Depends(
     scope: Scope = None,
     share: bool = True,
     wire: bool = True,
-    autowire: bool = True,
     sync_to_thread: bool = False,
 ) -> DependencyType:
     ...
@@ -35,7 +35,6 @@ def Depends(
     scope: Scope = None,
     share: bool = True,
     wire: bool = True,
-    autowire: bool = True,
     sync_to_thread: bool = False,
 ) -> DependencyType:
     ...
@@ -48,7 +47,6 @@ def Depends(
     scope: Scope = None,
     share: bool = True,
     wire: bool = True,
-    autowire: bool = True,
     sync_to_thread: bool = False,
 ) -> DependencyType:
     ...
@@ -61,7 +59,6 @@ def Depends(
     scope: Scope = None,
     share: bool = True,
     wire: bool = True,
-    autowire: bool = True,
     sync_to_thread: bool = False,
 ) -> DependencyType:
     ...
@@ -73,7 +70,6 @@ def Depends(
     scope: Scope = None,
     share: bool = True,
     wire: bool = True,
-    autowire: bool = True,
     sync_to_thread: bool = False,
 ) -> DependencyType:
     return Dependant(  # type: ignore[return-value]
@@ -81,6 +77,5 @@ def Depends(
         scope=scope,
         share=share,
         wire=wire,
-        autowire=autowire,
         sync_to_thread=sync_to_thread,
     )
